defmodule Slax.Chat do
  @moduledoc """
  Context to handle with chat details.
  """

  import Ecto.Query

  alias Slax.Chat.Message
  alias Slax.Chat.Room
  alias Slax.Repo

  def create_room(attrs) do
    %Room{}
    |> change_room(attrs)
    |> Repo.insert()
  end

  def update_room(%Room{} = room, attrs) do
    room
    |> Room.changeset(attrs)
    |> Repo.update()
  end

  def change_room(room, attrs \\ %{}) do
    Room.changeset(room, attrs)
  end

  def get_first_room! do
    query = from(r in Room, limit: 1, order_by: [asc: :name])
    Repo.one!(query)
  end

  def get_room!(id) do
    Repo.get!(Room, id)
  end

  def list_rooms do
    query = from(r in Room, order_by: [asc: :name])
    Repo.all(query)
  end

  def list_messages_in_room(%Room{id: room_id}) do
    query =
      from(m in Message,
        inner_join: u in assoc(m, :user),
        preload: [user: u],
        where: m.room_id == ^room_id,
        order_by: [asc: m.inserted_at, asc: m.id]
      )

    Repo.all(query)
  end

  def create_message(room, user, attrs) do
    %Message{room: room, user: user}
    |> change_message(attrs)
    |> Repo.insert()
  end

  def change_message(message, attrs \\ %{}) do
    Message.changeset(message, attrs)
  end
end
